@@ -4,9 +4,11 @@ use arrow::{
     array::{
         Array, ArrayRef, BinaryArray, BooleanArray, Date32Array, DecimalArray,
         FixedSizeBinaryArray, Float32Array, Int16Array, Int32Array, Int64Array, Int8Array,
-        StringArray, TimestampMillisecondArray, TimestampNanosecondArray, UInt8Array,
+        StringArray, TimestampMicrosecondArray, TimestampMillisecondArray,
+        TimestampNanosecondArray, UInt8Array,
     },
     datatypes::{DataType, Field, Schema},
+    record_batch::RecordBatchReader,
 };
 use chrono::NaiveDate;
 use float_eq::assert_float_eq;
@@ -14,6 +16,7 @@ use lazy_static::lazy_static;
 
 use arrow_odbc::{
     arrow::array::Float64Array,
+    arrow_schema_from,
     odbc_api::{
         sys::{AttrConnectionPooling, AttrCpMatch},
         Connection, Environment,
@@ -335,7 +338,36 @@ fn fetch_non_null_date_time() {
 
 /// Fill a record batch of nullable timestamps with milliseconds precision
 #[test]
-fn fetch_date_time() {
+fn fetch_date_time_us() {
+    let table_name = function_name!().rsplit_once(':').unwrap().1;
+
+    let array_any = fetch_arrow_data(
+        table_name,
+        "DATETIME2(6)",
+        "('2021-04-09 18:57:50'),(NULL),('2002-09-30 12:43:17')",
+    )
+    .unwrap();
+
+    let array_vals = array_any
+        .as_any()
+        .downcast_ref::<TimestampMicrosecondArray>()
+        .unwrap();
+
+    // Assert that the correct values are found within the arrow batch
+    assert_eq!(
+        Some(NaiveDate::from_ymd(2021, 4, 9).and_hms_milli(18, 57, 50, 0)),
+        array_vals.value_as_datetime(0)
+    );
+    assert!(array_vals.is_null(1));
+    assert_eq!(
+        Some(NaiveDate::from_ymd(2002, 9, 30).and_hms_milli(12, 43, 17, 00)),
+        array_vals.value_as_datetime(2)
+    );
+}
+
+/// Fill a record batch of nullable timestamps with milliseconds precision
+#[test]
+fn fetch_date_time_ms() {
     let table_name = function_name!().rsplit_once(':').unwrap().1;
 
     let array_any = fetch_arrow_data(
@@ -364,7 +396,7 @@ fn fetch_date_time() {
 
 /// Fill a record batch of non nullable timestamps with nanoseconds precision
 #[test]
-fn fetch_non_null_date_time_2() {
+fn fetch_non_null_date_time_ns() {
     let table_name = function_name!().rsplit_once(':').unwrap().1;
 
     let array_any = fetch_arrow_data(
@@ -521,6 +553,60 @@ fn prepared_query() {
     assert_float_eq!(&[1., 2., 3.][..], array_vals.values(), abs_all <= 000.1);
 }
 
+#[test]
+fn infer_schema() {
+    let table_name = function_name!().rsplit_once(':').unwrap().1;
+
+    // Setup a table on the database with some floats (so we can fetch them)
+    let conn = ENV.connect_with_connection_string(MSSQL).unwrap();
+    setup_empty_table(&conn, table_name, &["REAL NOT NULL"]).unwrap();
+
+    // Prepare query to get metadata
+    let sql = format!("SELECT a FROM {}", table_name);
+    let cursor = conn.execute(&sql, ()).unwrap().unwrap();
+
+    // Now that we have a cursor, we want to iterate over its rows and fill an arrow batch with it.
+
+    // Batches will contain at most 1 entries.
+    let max_batch_size = 1;
+
+    // Instantiate reader with Arrow schema and ODBC cursor
+    let reader = OdbcReader::new(cursor, max_batch_size).unwrap();
+
+    let actual = reader.schema();
+    let expected = Arc::new(Schema::new(vec![Field::new("a", DataType::Float32, false)]));
+
+    assert_eq!(expected, actual)
+}
+
+#[test]
+fn fetch_schema_for_table() {
+    let table_name = function_name!().rsplit_once(':').unwrap().1;
+
+    // Setup a table on the database with some floats (so we can fetch them)
+    let conn = ENV.connect_with_connection_string(MSSQL).unwrap();
+    setup_empty_table(&conn, table_name, &["REAL NOT NULL"]).unwrap();
+
+    // Prepare query to get metadata
+    let sql = format!("SELECT a FROM {}", table_name);
+    let prepared = conn.prepare(&sql).unwrap();
+
+    // Now that we have prepared statement, we want to use it to query metadata.
+    let schema = arrow_schema_from(&prepared).unwrap();
+
+    assert_eq!(
+        "Field { \
+            name: \"a\", \
+            data_type: Float32, \
+            nullable: false, \
+            dict_id: 0, \
+            dict_is_ordered: false, \
+            metadata: None \
+        }",
+        schema.to_string()
+    )
+}
+
 /// Inserts the values in the literal into the database and returns them as an Arrow array.
 fn fetch_arrow_data(
     table_name: &str,
